/* --------------------------------------------------------------
 * programme "BAL_8_Cantons_Monosens.ino"
 * --------------------------------------------------------------
 * ce programme gere un B.A.L de 8 cantons avec 8 signaux a
 * CATHODES COMMUNES. (Signaux a anodes communes possibles)
 * la commande du relais x par INx coupe l alimentation de la ZAx
 *  _______________________________________________
 * |                                               |
 * | C EST UN ETAT LOW QUI COMMANDE LE RELAIS !!!  |
 * |_______________________________________________|
 * 
 * Relais de 9 à 2 - Signaux luimineux de 22 à 45
 * Relais 1 : D09 -> IN1 - ZA1 => Relais 8 : D02 -> IN8 - ZA8 
 * Signaux sur sorties qui se suivent ordre vert - orange - rouge
 * ------------------------------------------------------------*/

const byte etatLedON = HIGH;  // Mettre LOW si signaux a anodes communes
const byte etatLedOFF = LOW; // Mettre HIGH si signaux a anodes communes

#include <PinChangeInt.h>

// definition des ILS
// ------------------
const byte pinILS1 = A0;
const byte pinILS2 = A1;
const byte pinILS3 = A2;
const byte pinILS4 = A3;
const byte pinILS5 = A4;
const byte pinILS6 = A5;
const byte pinILS7 = A6;
const byte pinILS8 = A7;

// Commandes des relais (etat LOW sur INx excite relais x)
// -------------------------------------------------------
const byte R1 = 9;
const byte R2 = 8;
const byte R3 = 7;
const byte R4 = 6;
const byte R5 = 5;
const byte R6 = 4;
const byte R7 = 3;
const byte R8 = 2;

// Les signaux de B.A.L
// --------------------
// Signal 1
const byte S1_vert = 22;
const byte S1_orange = 23;
const byte S1_rouge = 24;

// Signal 2
const byte S2_vert = 25;
const byte S2_orange = 26;
const byte S2_rouge = 27;

// Signal 3
const byte S3_vert = 28;
const byte S3_orange = 29;
const byte S3_rouge = 30;

// Signal 4
const byte S4_vert = 31;
const byte S4_orange = 32;
const byte S4_rouge = 33;

 // Signal 5
const byte S5_vert = 34;
const byte S5_orange = 35;
const byte S5_rouge = 36;

// Signal 6
const byte S6_vert = 37;
const byte S6_orange = 38;
const byte S6_rouge = 39;

// Signal 7
const byte S7_vert = 40;
const byte S7_orange = 41;
const byte S7_rouge = 42;

// Signal 8
const byte S8_vert = 43;
const byte S8_orange = 44;
const byte S8_rouge = 45;

// Occupation des cantons
// ----------------------
volatile bool canton1 = false;  // true si occupe
volatile bool canton2 = false;
volatile bool canton3 = false;
volatile bool canton4 = false;
volatile bool canton5 = false;
volatile bool canton6 = false;
volatile bool canton7 = false;
volatile bool canton8 = false;
 
const unsigned long dureeAntiRebond = 1;

// Les routines d'interruption
// ---------------------------
void interruptILS1()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton1 = true;
    canton8 = false;
    dateDernierChangement = date;
  }
}
 
void interruptILS2()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton2 = true;
    canton1 = false;
    dateDernierChangement = date;
  }
}
 
void interruptILS3()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton3 = true;
    canton2 = false;
    dateDernierChangement = date;
  }
}
 
void interruptILS4()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton4 = true;
    canton3 = false;
    dateDernierChangement = date;
  }
}

void interruptILS5()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton5 = true;
    canton4 = false;
    dateDernierChangement = date;
  }
}
 
void interruptILS6()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton6 = true;
    canton5 = false;
    dateDernierChangement = date;
  }
}
 
void interruptILS7()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton7 = true;
    canton6 = false;
    dateDernierChangement = date;
  }
}
 
void interruptILS8()
{
  static unsigned long dateDernierChangement = 0;
  
  unsigned long date = millis();
  if ((date - dateDernierChangement) > dureeAntiRebond) {
    canton8 = true;
    canton7 = false;
    dateDernierChangement = date;
  }
}

void setup()
{
  // Broches des ILS en entree et active le pullup interne
   pinMode(pinILS1, INPUT_PULLUP);
   pinMode(pinILS2, INPUT_PULLUP);
   pinMode(pinILS3, INPUT_PULLUP);
   pinMode(pinILS4, INPUT_PULLUP);
   pinMode(pinILS5, INPUT_PULLUP);
   pinMode(pinILS6, INPUT_PULLUP);
   pinMode(pinILS7, INPUT_PULLUP);
   pinMode(pinILS8, INPUT_PULLUP);

  // Programmes les broches relais en sortie
   for(int i = 2; i <= 9; i++) {
    pinMode(i, OUTPUT);
   }

  // Met les broches relais à HIGH (relais non excites)
   for(int i = 2; i <= 9; i++) {
    digitalWrite(i, HIGH); 
   }
   
  // Programme les feux des signaux en sortie
   for(int j = 22; j <= 45; j++){
     pinMode(j, OUTPUT);
   }

  // Eteint tous les feux des signaux
   for(int j = 22; j <= 45; j++){
     digitalWrite(j, LOW);
   }

  // Allume les huit signaux au vert au debut du jeu
   digitalWrite(S1_vert, HIGH);
   digitalWrite(S2_vert, HIGH);
   digitalWrite(S3_vert, HIGH);
   digitalWrite(S4_vert, HIGH);
   digitalWrite(S5_vert, HIGH);
   digitalWrite(S6_vert, HIGH);
   digitalWrite(S7_vert, HIGH);
   digitalWrite(S8_vert, HIGH);

  // Accroche les ISR aux pins
   PCintPort::attachInterrupt(pinILS1, interruptILS1, FALLING);
   PCintPort::attachInterrupt(pinILS2, interruptILS2, FALLING);
   PCintPort::attachInterrupt(pinILS3, interruptILS3, FALLING);
   PCintPort::attachInterrupt(pinILS4, interruptILS4, FALLING);
   PCintPort::attachInterrupt(pinILS5, interruptILS5, FALLING);
   PCintPort::attachInterrupt(pinILS6, interruptILS6, FALLING);
   PCintPort::attachInterrupt(pinILS7, interruptILS7, FALLING);
   PCintPort::attachInterrupt(pinILS8, interruptILS8, FALLING);
}  // Fin de setup
 
void loop()
{
  // Traitement du signal 1
  if(canton1){ // Signal 1 au rouge
    digitalWrite(S1_vert, etatLedOFF);
    digitalWrite(S1_orange, etatLedOFF);
    digitalWrite(S1_rouge, etatLedON); 
    // ZA precedente stop
    digitalWrite(R8, LOW);
  }
  if(!canton1 && canton2){ // Signal 1 a l orange
    digitalWrite(S1_vert, etatLedOFF);
    digitalWrite(S1_rouge, etatLedOFF); 
    // ZA precedente go
    digitalWrite(R8, HIGH);
    digitalWrite(S1_orange, etatLedON);    
  }
  if(!canton1 && !canton2){ // Signal 1 au vert
    digitalWrite(S1_orange, etatLedOFF);
    digitalWrite(S1_rouge, etatLedOFF);
    digitalWrite(S1_vert, etatLedON);    
  }

  // Traitement du signal 2
  if(canton2){ // Signal 2 au rouge
    digitalWrite(S2_vert, etatLedOFF);
    digitalWrite(S2_orange, etatLedOFF);
    digitalWrite(S2_rouge, etatLedON); 
    // ZA1 stop
    digitalWrite(R1, LOW);
  }
  if(!canton2 && canton3){ // Signal 2 a l orange
    digitalWrite(S2_vert, etatLedOFF);
    digitalWrite(S2_rouge, etatLedOFF); 
    // ZA1 go
    digitalWrite(R1, HIGH);
    digitalWrite(S2_orange, etatLedON);    
  }
  if(!canton2 && !canton3){ // Signal 2 au vert
    digitalWrite(S2_orange, etatLedOFF);
    digitalWrite(S2_rouge, etatLedOFF);
    digitalWrite(S2_vert, etatLedON);    
  }

  // Traitement du signal 3
  if(canton3){ // Signal 3 au rouge
    digitalWrite(S3_vert, etatLedOFF);
    digitalWrite(S3_orange, etatLedOFF);
    digitalWrite(S3_rouge, etatLedON); 
    // ZA2 stop
    digitalWrite(R2, LOW);
  }
  if(!canton3 && canton4){ // Signal 3 a l orange
    digitalWrite(S3_vert, etatLedOFF);
    digitalWrite(S3_rouge, etatLedOFF); 
    // ZA2 go
    digitalWrite(R2, HIGH);
    digitalWrite(S3_orange, etatLedON);    
  }
  if(!canton3 && !canton4){ // Signal 3 au vert
    digitalWrite(S3_orange, etatLedOFF);
    digitalWrite(S3_rouge, etatLedOFF);
    digitalWrite(S3_vert, etatLedON);    
  }

  // Traitement du signal 4
  if(canton4){ // Signal 4 au rouge
    digitalWrite(S4_vert, etatLedOFF);
    digitalWrite(S4_orange, etatLedOFF);
    digitalWrite(S4_rouge, etatLedON); 
    // ZA3 stop
    digitalWrite(R3, LOW);
  }
  if(!canton4 && canton1){ // Signal 4 a l orange
    digitalWrite(S4_vert, etatLedOFF);
    digitalWrite(S4_rouge, etatLedOFF); 
    // ZA3 go
    digitalWrite(R3, HIGH);
    digitalWrite(S4_orange, etatLedON);    
  }
  if(!canton4 && !canton1){ // Signal 4 au vert
    digitalWrite(S4_orange, etatLedOFF);
    digitalWrite(S4_rouge, etatLedOFF);
    digitalWrite(S4_vert, etatLedON);    
  }

  // Traitement du signal 5
  if(canton5){ // Signal 5 au rouge
    digitalWrite(S5_vert, etatLedOFF);
    digitalWrite(S5_orange, etatLedOFF);
    digitalWrite(S5_rouge, etatLedON); 
    // ZA4 stop
    digitalWrite(R4, LOW);
  }
  if(!canton5 && canton6){ // Signal 5 a l orange
    digitalWrite(S5_vert, etatLedOFF);
    digitalWrite(S5_rouge, etatLedOFF); 
    // ZA4 go
    digitalWrite(R4, HIGH);
    digitalWrite(S5_orange, etatLedON);    
  }
  if(!canton5 && !canton1){ // Signal 5 au vert
    digitalWrite(S5_orange, etatLedOFF);
    digitalWrite(S5_rouge, etatLedOFF);
    digitalWrite(S5_vert, etatLedON);    
  }

  // Traitement du signal 6
  if(canton6){ // Signal 6 au rouge
    digitalWrite(S6_vert, etatLedOFF);
    digitalWrite(S6_orange, etatLedOFF);
    digitalWrite(S6_rouge, etatLedON); 
    // ZA5 stop
    digitalWrite(R5, LOW);
  }
  if(!canton6 && canton7){ // Signal 6 a l orange
    digitalWrite(S6_vert, etatLedOFF);
    digitalWrite(S6_rouge, etatLedOFF); 
    // ZA5 go
    digitalWrite(R5, HIGH);
    digitalWrite(S6_orange, etatLedON);    
  }
  if(!canton6 && !canton7){ // Signal 6 au vert
    digitalWrite(S6_orange, etatLedOFF);
    digitalWrite(S6_rouge, etatLedOFF);
    digitalWrite(S6_vert, etatLedON);    
  }

  // Traitement du signal 7
  if(canton7){ // Signal 7 au rouge
    digitalWrite(S7_vert, etatLedOFF);
    digitalWrite(S7_orange, etatLedOFF);
    digitalWrite(S7_rouge, etatLedON); 
    // ZA6 stop
    digitalWrite(R6, LOW);
  }
  if(!canton7 && canton8){ // Signal 7 a l orange
    digitalWrite(S8_vert, etatLedOFF);
    digitalWrite(S8_rouge, etatLedOFF); 
    // ZA6 go
    digitalWrite(R6, HIGH);
    digitalWrite(S8_orange, etatLedON);    
  }
  if(!canton7 && !canton8){ // Signal 7 au vert
    digitalWrite(S7_orange, etatLedOFF);
    digitalWrite(S7_rouge, etatLedOFF);
    digitalWrite(S7_vert, etatLedON);    
  }

  // Traitement du signal 8
  if(canton8){ // Signal 8 au rouge
    digitalWrite(S8_vert, etatLedOFF);
    digitalWrite(S8_orange, etatLedOFF);
    digitalWrite(S8_rouge, etatLedON); 
    // ZA7 stop
    digitalWrite(R7, LOW);
  }
  if(!canton8 && canton1){ // Signal 8 a l orange
    digitalWrite(S8_vert, etatLedOFF);
    digitalWrite(S8_rouge, etatLedOFF); 
    // ZA7 go
    digitalWrite(R7, HIGH);
    digitalWrite(S8_orange, etatLedON);    
  }
  if(!canton8 && !canton1){ // Signal 8 au vert
    digitalWrite(S8_orange, etatLedOFF);
    digitalWrite(S8_rouge, etatLedOFF);
    digitalWrite(S8_vert, etatLedON);    
  }

}  // Fin de loop
